# Generated by Django 4.0.10 on 2024-06-23 14:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("structures", "0005_add_notification_types"),
    ]

    operations = [
        migrations.AddField(
            model_name="ownercharacter",
            name="disabled_reason",
            field=models.TextField(default=""),
        ),
        migrations.AddField(
            model_name="ownercharacter",
            name="is_enabled",
            field=models.BooleanField(
                default=True,
                help_text="Disabled characters are not used for syncing owners",
                verbose_name="is enabled",
            ),
        ),
    ]
