@media all {

    /* General */
    h3 {
        margin-top: 0px;
    }

    .floating-icon {
        float: left;
        width: 48px;
        height: 48px;
        margin-right: 0.75em;
    }

    .center-image {
        display: block;
        margin-left: auto;
        margin-right: auto;
    }

}
